//***********************************************************************
// Software that is described herein is for illustrative purposes only
// which provides customers with programming information regarding the
// products. This software is supplied "AS IS" without any warranties.
// NXP Semiconductors assumes no responsibility or liability for the
// use of the software, conveys no license or title under any patent,
// copyright, or mask work right to the product. NXP Semiconductors
// reserves the right to make changes in the software without
// notification. NXP Semiconductors also make no representation or
// warranty that such application will be suitable for the specified
// use without further testing or modification.
// ********************************************************************


unit LPC214x;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, ExtCtrls, StdCtrls, Spin, JvHidControllerClass, Buttons, ComCtrls, Math;

const
  cNXP_VID = $04CC;
  cLPC_USB_PID= $1235;

type
  TLPC2140CReport = packed record
    ReportID: Byte; // all reports have a ReportID
    LEDBits: Byte;  // 8 bits to read and write
  end;

type
  TfrmMain = class(TForm)
    lblHIDstatus: TStaticText;
    Panel1: TPanel;
    StaticText1: TStaticText;
    Button1: TButton;
    Button2: TButton;
    Button3: TButton;
    Label1: TLabel;
    Label2: TLabel;
    Label3: TLabel;
    Button5: TButton;
    Label4: TLabel;
    LED0: TCheckBox;
    HidCtl: TJvHidDeviceController;
    procedure LEDClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure HidCtlDeviceChange(Sender: TObject);
    function HidCtlEnumerate(const HidDev: TJvHidDevice; const Idx: Integer): Boolean;
    procedure UpdateControls;
    procedure FormActivate(Sender: TObject);
    procedure HidCtlDeviceData(HidDev: TJvHidDevice; ReportID: Byte;
      const Data: Pointer; Size: Word);
    procedure Button1Click(Sender: TObject);
    procedure Button2Click(Sender: TObject);
    procedure Button3Click(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure Button5Click(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
    LPC2140_HID: TJvHidDevice;
    LPC2140_OutputReport: TLPC2140CReport;
    LED: array [0..7] of TCheckBox;
  end;

var
  frmMain: TfrmMain;
  Port0Value: byte = $00;
  Shift: boolean;
implementation

{$R *.dfm}

procedure TfrmMain.LEDClick(Sender: TObject);
var
BytesWritten: Cardinal;
begin
if (Sender As TCheckBox).Checked
  then Port0Value := Port0Value OR (Sender As TCheckBox).Tag
  else Port0Value := Port0Value AND NOT(Sender As TCheckBox).Tag;

LPC2140_OutputReport.LEDBits := Port0Value;
LPC2140_HID.WriteFile(LPC2140_OutputReport, SizeOf(LPC2140_OutputReport), BytesWritten);
end;

procedure TfrmMain.FormCreate(Sender: TObject);
var
i: integer;
begin
LED[0] := LED0;

for i := 1 to 7 do
    begin
    LED[i] := TCheckBox.Create(Self);
    LED[i].Width := LED0.Width;
    LED[i].Caption := 'P1.' + IntToStr(i+16);
    LED[i].Left := LED0.Left;
    LED[i].Top := LED0.Top + i*16;
    if not Assigned(LED[i].Parent) then
      LED[i].Parent := LED0.Parent;
    LED[i].TabOrder := LED0.TabOrder + i;
    LED[i].Tag := 1 SHL i;
    end
end;

procedure TfrmMain.HidCtlDeviceChange(Sender: TObject);
begin
// Free the device object if it has been unplugged
if Assigned(LPC2140_HID) and not LPC2140_HID.IsPluggedIn then
    begin
    LPC2140_HID.Free;
    LPC2140_HID := nil;
    end;
  // if no LPC2140_HID in use yet then search for one
  if not Assigned(LPC2140_HID) then
    HidCtl.Enumerate;
  // update the controls on the form
  UpdateControls;
end;

function TfrmMain.HidCtlEnumerate(const HidDev: TJvHidDevice; const Idx: Integer): Boolean;
var
  BytesWritten: Cardinal;
begin
  if (HidDev.Attributes.VendorID = cNXP_VID) and
     (HidDev.Attributes.ProductID = cLPC_USB_PID) then
  begin
    HidCtl.CheckOutByIndex(LPC2140_HID, Idx);
    // initialize the output report
    LPC2140_OutputReport.ReportID := 0;
    LPC2140_OutputReport.LEDBits := $00;
    // write the bits to the LPC2140_HID to reset the LEDs
    LPC2140_HID.WriteFile(LPC2140_OutputReport, SizeOf(LPC2140_OutputReport), BytesWritten);
  end;

  // stop enumeration if not found
  Result := not Assigned(LPC2140_HID);
end;

procedure TfrmMain.UpdateControls;
var
i: Integer;
begin
  // OnDeviceChange happens before OnCreate so the LEDs are not assigned yet
  if Assigned(LED[Low(LED)]) then
  begin
    // enable/disable the controls
    for i := Low(LED) to High(LED) do
      LED[i].Enabled := Assigned(LPC2140_HID);
   if Assigned(LPC2140_HID) then
      begin
      lblHIDstatus.Caption := 'MCB2140 Board detected';
      lblHIDstatus.Color := clGreen;
      Button1.Enabled := true;
      Button2.Enabled := true;
      Button3.Enabled := true;
      Button5.Enabled := true;
      end
   else
      begin
      lblHIDstatus.Caption := 'MCB2140 Board not detected';
      lblHIDstatus.Color := clRed;
      for i:=0 to 7 do
        begin
        LED[i].OnClick := nil;
        LED[i].Checked := false;
        LED[i].OnClick := LEDClick;
        end;
        Button1.Enabled := false;
        Button2.Enabled := false;
        Button3.Enabled := false;
        Button5.Enabled := false;
      end;
  end;
end;

procedure TfrmMain.FormActivate(Sender: TObject);
begin
UpdateControls;
end;

procedure TfrmMain.HidCtlDeviceData(HidDev: TJvHidDevice; ReportID: Byte;
  const Data: Pointer; Size: Word);
begin

if StrPas(Data)<>'' then
    begin
    StaticText1.Caption := 'INT1 pressed';
    StaticText1.Color := clRed;
    StaticText1.Font.Color := clCream;
    end
    else
    begin
    StaticText1.Caption := 'INT1 not pressed';
    StaticText1.Color := clCream;
    StaticText1.Font.Color := clNavy;
    end;
end;

procedure TfrmMain.Button1Click(Sender: TObject);
begin
Label1.Caption := LPC2140_HID.VendorName;
end;

procedure TfrmMain.Button2Click(Sender: TObject);
begin
Label2.Caption := LPC2140_HID.ProductName;
end;

procedure TfrmMain.Button3Click(Sender: TObject);
begin
Label3.Caption := LPC2140_HID.SerialNumber;
end;

procedure TfrmMain.Button5Click(Sender: TObject);
begin
Label4.Caption := LPC2140_HID.DeviceStrings[$56];
end;

procedure TfrmMain.FormDestroy(Sender: TObject);
begin
if Assigned(LPC2140_HID) then LPC2140_HID.Destroy;
end;

end.
